﻿using Kivii;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Jst.Report.Download
{
    public partial class FrmMain : Form
    {
        #region 全局变量
        private JsonServiceClient _client = null;

        private int _take = 200;

        private int _skip = 0;

		private RestfulQueryResponse<Report> _results = null;

        private bool _groupByProductJudgement = false;

        private bool _groupByProductBrand = false;

        private bool _renameFileByBatchNumber = false;

        private bool _renameFileByWph = false;

        private bool _renameFileByAjt = false;

        private string _fileRootPath = string.Empty;
        #endregion

        public FrmMain()
        {
            InitializeComponent();
        }

        private void FrmMain_Load(object sender, EventArgs e)
        {
            dgvReports.AutoGenerateColumns = false;
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            InitClient();
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                lblMessage.Text = $"未连接";
                return;
            }
            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(res =>
            {
                _client.Dispose();
                _client = null;
                lblMessage.Text = $"退出成功";
            });
            task.Error(err =>
            {
                _client.Dispose();
                _client = null;
                lblMessage.Text = $"已强制退出";
            });
        }

        private void InitClient()
        {
            _client = new JsonServiceClient(tbxUrlBase.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(res =>
            {
                lblMessage.Text = $"登录成功";
            });
            task.Error(err =>
            {
                lblMessage.Text = $"登录错误:{err.Message}";
                _client.Dispose();
                _client = null;
            }, true);
        }

        #region 日期选择
        private void dtpDealDateBegin_ValueChanged(object sender, EventArgs e)
        {
            dtpValueChange(sender);
        }

        private void dtpDealDateFinish_ValueChanged(object sender, EventArgs e)
        {
            dtpValueChange(sender);
        }

        private void dtpDeadDateBegin_ValueChanged(object sender, EventArgs e)
        {
            dtpValueChange(sender);
        }

        private void dtpDeadDateFinish_ValueChanged(object sender, EventArgs e)
        {
            dtpValueChange(sender);
        }

        private void dtpReportIssueDateBegin_ValueChanged(object sender, EventArgs e)
        {
            dtpValueChange(sender);
        }

        private void dtpReportIssueDateFinish_ValueChanged(object sender, EventArgs e)
        {
            dtpValueChange(sender);
        }

        public void dtbKeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)8)  // backspace左删除键
            {
                DateTimePicker dtp = (DateTimePicker)sender;
                dtp.Format = DateTimePickerFormat.Custom;
                dtp.CustomFormat = " ";
            }
        }

        private void dtpValueChange(object sender)
        {
            DateTimePicker dtp = (DateTimePicker)sender;
            dtp.Format = DateTimePickerFormat.Long;
            dtp.CustomFormat = null; //null;
            dtp.Checked = false;
        }

        private void dtpDealDateBegin_KeyPress(object sender, KeyPressEventArgs e)
        {
            dtbKeyPress(sender, e);
        }

        private void dtpDealDateFinish_KeyPress(object sender, KeyPressEventArgs e)
        {
            dtbKeyPress(sender, e);
        }

        private void dtpDeadDateBegin_KeyPress(object sender, KeyPressEventArgs e)
        {
            dtbKeyPress(sender, e);
        }

        private void dtpDeadDateFinish_KeyPress(object sender, KeyPressEventArgs e)
        {
            dtbKeyPress(sender, e);
        }

        private void dtpReportIssueDateBegin_KeyPress(object sender, KeyPressEventArgs e)
        {
            dtbKeyPress(sender, e);
        }

        private void dtpReportIssueDateFinish_KeyPress(object sender, KeyPressEventArgs e)
        {
            dtbKeyPress(sender, e);
        }
        #endregion

        #region 查询
        private void btnSearch_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                lblMessage.Text = $"查询错误:请先登录!";
                return;
            }
            btnPreviewPage.Enabled = false;
            btnNextPage.Enabled = false;
            btnDownload.Enabled = true;
            queryContract();
        }

        private void btnPreviewPage_Click(object sender, EventArgs e)
        {
            btnPreviewPage.Enabled = false;
            int num = _skip - _take;
            if (num < 0)
            {
                num = 0;
            }
            queryContract(num);
        }

        private void btnNextPage_Click(object sender, EventArgs e)
        {
            btnNextPage.Enabled = false;
            int skip = _skip + _take;
            queryContract(skip);
        }
        private void queryContract(int skip = 0)
        {
            btnSearch.Enabled = false;
            lblMessage.Text = "查询中";
            ReportQuery contractQuery = new ReportQuery();
            if (!tbxReportIdContains.Text.IsNullOrEmpty())
            {
                contractQuery.ReportIdContains = tbxReportIdContains.Text;
            }
            if (!tbxCustomerOrgNameContains.Text.IsNullOrEmpty())
            {
                contractQuery.DemanderNameContains = tbxCustomerOrgNameContains.Text;
            }
            if (!tbxProductOrgNameContains.Text.IsNullOrEmpty())
            {
                contractQuery.SampleManufacturerNameContains = tbxProductOrgNameContains.Text;
            }
            if (!tbxPlanNumber.Text.IsNullOrEmpty())
            {
                contractQuery.SampleSourceBatchNumber = tbxPlanNumber.Text;
            }
            if (!tbxCustomSuffixId.Text.IsNullOrEmpty())
            {
                contractQuery.Category = tbxCustomSuffixId.Text;
            }
            if (dtpDealDateBegin.Text != " ")
            {
                contractQuery.DealDateGreaterThanOrEqualTo = DateTime.Parse(dtpDealDateBegin.Value.ToShortDateString().ToString());
            }
            if (dtpDealDateFinish.Text != " ")
            {
                contractQuery.DealDateLessThanOrEqualTo = DateTime.Parse(dtpDealDateFinish.Value.AddDays(1.0).ToShortDateString().ToString());
            }
            if (dtpDeadDateBegin.Text != " ")
            {
                contractQuery.DeadDateGreaterThanOrEqualTo = DateTime.Parse(dtpDeadDateBegin.Value.ToShortDateString().ToString());
            }
            if (dtpDeadDateFinish.Text != " ")
            {
                contractQuery.DeadDateLessThanOrEqualTo = DateTime.Parse(dtpDeadDateFinish.Value.AddDays(1.0).ToShortDateString().ToString());
            }
            if (dtpReportIssueDateBegin.Text != " ")
            {
                contractQuery.IssueDateGreaterThanOrEqualTo = DateTime.Parse(dtpReportIssueDateBegin.Value.ToShortDateString().ToString());
            }
            if (dtpReportIssueDateFinish.Text != " ")
            {
                contractQuery.IssueDateLessThanOrEqualTo = DateTime.Parse(dtpReportIssueDateFinish.Value.AddDays(1.0).ToShortDateString().ToString());
            }
            //contractQuery.DealDateGreaterThan = dtpDealDateBegin.Value;			
            //contractQuery.DealDateLessThan = dtpDealDateFinish.Value.AddDays(1.0);
            //contractQuery.DeadDateGreaterThan = dtpDeadDateBegin.Value;
            //contractQuery.DeadDateLessThan = dtpDeadDateFinish.Value.AddDays(1.0);
            _take = tbxTake.Text.ToInt(_take);
            if (_take > 200)
            {
                _take = 200;
            }
            _skip = skip;
            tbxTake.Text = _take.ToString();
            contractQuery.Take = _take;
            contractQuery.Skip = _skip;
            Task<RestfulQueryResponse<Report>> task = _client.PostAsync(contractQuery);
            ((Task)task).Success((Action)delegate
            {
                btnSearch.Enabled = true;
                _results = task.Result;
                dgvReports.DataSource = _results.Results;
                lblMessage.Text = $"查询成功:共{_results.Total}条记录,正显示{_skip + 1}至{skip + _results.Results.Count}条数据";
                setPageButtons();
            }, onUiThread: true, TaskContinuationOptions.OnlyOnRanToCompletion);
            task.Error(delegate (Exception ex)
            {
                btnSearch.Enabled = true;
                btnPreviewPage.Enabled = false;
                btnNextPage.Enabled = false;
                lblMessage.Text = $"查询错误:{ex.Message}";
            });
        }
        private void setPageButtons()
        {
            btnPreviewPage.Enabled = (_skip != 0);
            btnNextPage.Enabled = (_results.Total > _skip + _take);
        }
        #endregion

        #region 下载
        private void btnSelectDirectory_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                tbxFileSaveDirectory.Text = folderBrowserDialog.SelectedPath;
            }
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            if (tbxFileSaveDirectory.Text.IsNullOrEmpty())
            {
                MessageBox.Show("请先选择要保存的文件目录");
                return;
            }
            btnDownload.Enabled = false;
            _fileRootPath = tbxFileSaveDirectory.Text;
            Directory.CreateDirectory(_fileRootPath);
            string path = "";
            if (dtpDealDateBegin.Text != " " && dtpDealDateFinish.Text != " ")
            {
                path = dtpDealDateBegin.Value.ToString("yyyy-MM-dd") + "至" + dtpDealDateFinish.Value.ToString("yyyy-MM-dd");
            }
            else if (dtpDeadDateBegin.Text != " " && dtpDeadDateFinish.Text != " ")
            {
                path = dtpDeadDateBegin.Value.ToString("yyyy-MM-dd") + "至" + dtpDeadDateFinish.Value.ToString("yyyy-MM-dd");
            }
            else if (dtpReportIssueDateBegin.Text != " " && dtpReportIssueDateFinish.Text != " ")
            {
                path = dtpReportIssueDateBegin.Value.ToString("yyyy-MM-dd") + "至" + dtpReportIssueDateFinish.Value.ToString("yyyy-MM-dd");
            }
            else
            {
                btnSearch.Enabled = true;
                btnPreviewPage.Enabled = false;
                btnNextPage.Enabled = false;
                lblMessage.Text = "下载失败:未设置任何起止日期条件，无法下载至对应的目录中！";
                return;
            }
            _fileRootPath = Path.Combine(_fileRootPath, path);
            Directory.CreateDirectory(_fileRootPath);
            _groupByProductJudgement = cbxGroupByProductJudgement.Checked;
            _groupByProductBrand = cbxGroupByProductBrand.Checked;
            _renameFileByBatchNumber = cbxRenameFileByBatchNumber.Checked;
            _renameFileByWph = cbxRenameFileByWph.Checked;
            _renameFileByAjt = cbxRenameFileByAjt.Checked;
            Thread thread = new Thread(download);
            thread.Start();
        }

        private void download()
        {
            int succeed2 = 0;
            int error2 = 0;
            int index2 = 0;
            WebClient webClient = new WebClient();
            foreach (Report result in _results.Results)
            {
                try
                {
                    int num = index2;
                    index2 = num + 1;
                    Invoke((Action)delegate
                    {
                        lblMessage.Text = $"正在下载报告{result.ReportId},共{_results.Results.Count}条，正在处理第{index2}条....";
                    });
                    string text = $"{result.ReportId}.pdf";
                    if (_renameFileByBatchNumber)
                    {
                        string text2 = result.SampleBatchNumber.Trim();
                        if (!text2.IsNullOrEmpty())
                        {
                            text = $"{result.ReportId}({text2}).pdf";
                        }
                    }
                    if (_renameFileByAjt)
                    {
                        string ReportId = result.ReportId.Trim();
                        string BatchNumber = result.SampleBatchNumber.Trim();
                        string ProductName = result.SampleName.Trim();
                        string ProductJudgement = result.TestJudgement.Trim();
                        if (ProductJudgement.IsNullOrEmpty() || ProductJudgement == "/" || ProductJudgement == "\\")
                        {
                            ProductJudgement = "不判定";
                        }
                        text = $"{ReportId} {BatchNumber}{ProductName}{ProductJudgement}.pdf";
                    }
                    if (_renameFileByWph)
                    {
                        string SamplingDate = null;
                        string BatchNumber = result.SampleBatchNumber.Trim();
                        string ProductBrand = result.SampleBrand.Trim();
                        string ProductName = result.SampleName.Trim();
                        string ProductJudgement = result.TestJudgement.Trim();
                        if (ProductJudgement.IsNullOrEmpty() || ProductJudgement == "/" || ProductJudgement == "\\")
                        {
                            ProductJudgement = "不判定";
                        }
                        try
                        {
                            SamplingDate = Convert.ToDateTime(result.Metadata["SamplingDate"]).ToString("MM.dd");
                        }
                        catch
                        {
                            SamplingDate = "无抽样日期";
                        }
                        text = $"{SamplingDate}+江苏纺检+{ProductBrand}+{ProductName}+{BatchNumber}+{ProductJudgement}.pdf";
                    }
                    text = text.Replace("\\", "_").Replace("/", "_").Replace("*", "_");
                    if (_groupByProductJudgement)
                    {
                        string text3 = result.TestJudgement.Trim();
                        if (text3.IsNullOrEmpty() || text3 == "/" || text3 == "\\")
                        {
                            text3 = "不判定";
                        }
                        text = Path.Combine(text3, text);
                    }
                    if (_groupByProductBrand)
                    {
                        string text4 = result.SampleBrand.Trim();
                        if (text4.IsNullOrEmpty())
                        {
                            text4 = "其它";
                        }
                        text = Path.Combine(text4, text);
                    }

                    text = Path.Combine(_fileRootPath, text);
                    string directoryName = Path.GetDirectoryName(text);
                    Directory.CreateDirectory(directoryName);
                    var code = $"{result.Kvid.ToString().Substring(0, 4)}/{result.ReportId}";
                    string address = $"http://k5.jst-gov.com/Lims/Report/{code}";
                    webClient.DownloadFile(address, text);
                    num = succeed2;
                    succeed2 = num + 1;
                }
                catch (Exception ex)
                {
                    int num = error2;
                    error2 = num + 1;
                }
            }
            Invoke((Action)delegate
            {
                lblMessage.Text = $"共成功下载{succeed2}个文件，失败了{error2}个";
                btnDownload.Enabled = true;
            });
        }
        #endregion

        
    }
}
